@@ -1,17 +1,26 @@
 # pylint: disable=W0223
 
-from pandas.core.index import Index, MultiIndex
+import numpy as np
 from pandas.compat import range, zip
 import pandas.compat as compat
+from pandas.types.generic import ABCDataFrame, ABCPanel, ABCSeries
+from pandas.types.common import (is_integer_dtype,
+                                 is_integer, is_float,
+                                 is_categorical_dtype,
+                                 is_list_like,
+                                 is_sequence,
+                                 is_scalar,
+                                 is_sparse,
+                                 _is_unorderable_exception,
+                                 _ensure_platform_int)
+from pandas.types.missing import isnull, _infer_fill_value
+
+from pandas.core.index import Index, MultiIndex
+
 import pandas.core.common as com
-import pandas.lib as lib
-from pandas.core.common import (is_bool_indexer, is_integer_dtype,
-                                _asarray_tuplesafe, is_list_like, isnull,
-                                is_null_slice, is_full_slice, ABCSeries,
-                                ABCDataFrame, ABCPanel, is_float,
-                                _values_from_object, _infer_fill_value,
-                                is_integer)
-import numpy as np
+from pandas.core.common import (is_bool_indexer, _asarray_tuplesafe,
+                                is_null_slice, is_full_slice,
+                                _values_from_object)
 
 
 # the supported indexers
@@ -45,36 +54,36 @@ class IndexingError(Exception):
 class _NDFrameIndexer(object):
     _valid_types = None
     _exception = KeyError
+    axis = None
 
     def __init__(self, obj, name):
         self.obj = obj
         self.ndim = obj.ndim
         self.name = name
-        self.axis = None
 
-    def __call__(self, *args, **kwargs):
+    def __call__(self, axis=None):
         # we need to return a copy of ourselves
-        self = self.__class__(self.obj, self.name)
+        new_self = self.__class__(self.obj, self.name)
 
-        # set the passed in values
-        for k, v in compat.iteritems(kwargs):
-            setattr(self, k, v)
-        return self
+        new_self.axis = axis
+        return new_self
 
     def __iter__(self):
         raise NotImplementedError('ix is not iterable')
 
     def __getitem__(self, key):
         if type(key) is tuple:
+            key = tuple(com._apply_if_callable(x, self.obj) for x in key)
             try:
                 values = self.obj.get_value(*key)
-                if lib.isscalar(values):
+                if is_scalar(values):
                     return values
             except Exception:
                 pass
 
             return self._getitem_tuple(key)
         else:
+            key = com._apply_if_callable(key, self.obj)
             return self._getitem_axis(key, axis=0)
 
     def _get_label(self, label, axis=0):
@@ -124,6 +133,10 @@ class _NDFrameIndexer(object):
             raise IndexingError(key)
 
     def __setitem__(self, key, value):
+        if isinstance(key, tuple):
+            key = tuple(com._apply_if_callable(x, self.obj) for x in key)
+        else:
+            key = com._apply_if_callable(key, self.obj)
         indexer = self._get_setitem_indexer(key)
         self._setitem_with_indexer(indexer, value)
 
@@ -332,9 +345,12 @@ class _NDFrameIndexer(object):
                     # this preserves dtype of the value
                     new_values = Series([value])._values
                     if len(self.obj._values):
-                        new_values = np.concatenate([self.obj._values,
-                                                     new_values])
-
+                        try:
+                            new_values = np.concatenate([self.obj._values,
+                                                         new_values])
+                        except TypeError:
+                            new_values = np.concatenate([self.obj.asobject,
+                                                         new_values])
                     self.obj._data = self.obj._constructor(
                         new_values, index=new_index, name=self.obj.name)._data
                     self.obj._maybe_update_cacher(clear=True)
@@ -618,7 +634,7 @@ class _NDFrameIndexer(object):
             # we have a frame, with multiple indexers on both axes; and a
             # series, so need to broadcast (see GH5206)
             if (sum_aligners == self.ndim and
-                    all([com.is_sequence(_) for _ in indexer])):
+                    all([is_sequence(_) for _ in indexer])):
                 ser = ser.reindex(obj.axes[0][indexer[0]], copy=True)._values
 
                 # single indexer
@@ -632,7 +648,7 @@ class _NDFrameIndexer(object):
                 ax = obj.axes[i]
 
                 # multiple aligners (or null slices)
-                if com.is_sequence(idx) or isinstance(idx, slice):
+                if is_sequence(idx) or isinstance(idx, slice):
                     if single_aligner and is_null_slice(idx):
                         continue
                     new_ix = ax[idx]
@@ -678,7 +694,7 @@ class _NDFrameIndexer(object):
 
                     return ser
 
-        elif lib.isscalar(indexer):
+        elif is_scalar(indexer):
             ax = self.obj._get_axis(1)
 
             if ser.index.equals(ax):
@@ -703,7 +719,7 @@ class _NDFrameIndexer(object):
             sindexers = []
             for i, ix in enumerate(indexer):
                 ax = self.obj.axes[i]
-                if com.is_sequence(ix) or isinstance(ix, slice):
+                if is_sequence(ix) or isinstance(ix, slice):
                     if idx is None:
                         idx = ax[ix].ravel()
                     elif cols is None:
@@ -754,7 +770,7 @@ class _NDFrameIndexer(object):
                 val = df.reindex(index=ax)._values
             return val
 
-        elif lib.isscalar(indexer) and is_panel:
+        elif is_scalar(indexer) and is_panel:
             idx = self.obj.axes[1]
             cols = self.obj.axes[2]
 
@@ -832,7 +848,7 @@ class _NDFrameIndexer(object):
                 [(a, self._convert_for_reindex(t, axis=o._get_axis_number(a)))
                  for t, a in zip(tup, o._AXIS_ORDERS)])
             return o.reindex(**d)
-        except:
+        except(KeyError, IndexingError):
             raise self._exception
 
     def _convert_for_reindex(self, key, axis=0):
@@ -850,7 +866,7 @@ class _NDFrameIndexer(object):
                 keyarr = _asarray_tuplesafe(key)
 
             if is_integer_dtype(keyarr) and not labels.is_integer():
-                keyarr = com._ensure_platform_int(keyarr)
+                keyarr = _ensure_platform_int(keyarr)
                 return labels.take(keyarr)
 
             return keyarr
@@ -888,7 +904,9 @@ class _NDFrameIndexer(object):
 
         # we maybe be using a tuple to represent multiple dimensions here
         ax0 = self.obj._get_axis(0)
-        if isinstance(ax0, MultiIndex):
+        # ...but iloc should handle the tuple as simple integer-location
+        # instead of checking it as multiindex representation (GH 13797)
+        if isinstance(ax0, MultiIndex) and self.name != 'iloc':
             result = self._handle_lowerdim_multi_index_axis0(tup)
             if result is not None:
                 return result
@@ -961,7 +979,7 @@ class _NDFrameIndexer(object):
             axis += 1
 
             # if we have a scalar, we are done
-            if lib.isscalar(obj) or not hasattr(obj, 'ndim'):
+            if is_scalar(obj) or not hasattr(obj, 'ndim'):
                 break
 
             # has the dim of the obj changed?
@@ -1031,7 +1049,7 @@ class _NDFrameIndexer(object):
                 # asarray can be unsafe, NumPy strings are weird
                 keyarr = _asarray_tuplesafe(key)
 
-            if com.is_categorical_dtype(labels):
+            if is_categorical_dtype(labels):
                 keyarr = labels._shallow_copy(keyarr)
 
             # have the index handle the indexer and possibly return
@@ -1203,7 +1221,9 @@ class _NDFrameIndexer(object):
                     else:
                         (indexer,
                          missing) = labels.get_indexer_non_unique(objarr)
-                        check = indexer
+                        # 'indexer' has dupes, create 'check' using 'missing'
+                        check = np.zeros_like(objarr)
+                        check[missing] = -1
 
                 mask = check == -1
                 if mask.any():
@@ -1280,6 +1300,12 @@ class _LocationIndexer(_NDFrameIndexer):
     _exception = Exception
 
     def __getitem__(self, key):
+        if isinstance(key, tuple):
+            key = tuple(com._apply_if_callable(x, self.obj) for x in key)
+        else:
+            # scalar callable may return tuple
+            key = com._apply_if_callable(key, self.obj)
+
         if type(key) is tuple:
             return self._getitem_tuple(key)
         else:
@@ -1328,6 +1354,8 @@ class _LocIndexer(_LocationIndexer):
     - A slice object with labels, e.g. ``'a':'f'`` (note that contrary
       to usual python slices, **both** the start and the stop are included!).
     - A boolean array.
+    - A ``callable`` function with one argument (the calling Series, DataFrame
+      or Panel) and that returns valid output for indexing (one of the above)
 
     ``.loc`` will raise a ``KeyError`` when the items are not found.
 
@@ -1384,7 +1412,7 @@ class _LocIndexer(_LocationIndexer):
             except TypeError as e:
 
                 # python 3 type errors should be raised
-                if 'unorderable' in str(e):  # pragma: no cover
+                if _is_unorderable_exception(e):
                     error()
                 raise
             except:
@@ -1392,8 +1420,33 @@ class _LocIndexer(_LocationIndexer):
 
         return True
 
+    def _get_partial_string_timestamp_match_key(self, key, labels):
+        """Translate any partial string timestamp matches in key, returning the
+        new key (GH 10331)"""
+        if isinstance(labels, MultiIndex):
+            if isinstance(key, compat.string_types) and \
+                    labels.levels[0].is_all_dates:
+                # Convert key '2016-01-01' to
+                # ('2016-01-01'[, slice(None, None, None)]+)
+                key = tuple([key] + [slice(None)] * (len(labels.levels) - 1))
+
+            if isinstance(key, tuple):
+                # Convert (..., '2016-01-01', ...) in tuple to
+                # (..., slice('2016-01-01', '2016-01-01', None), ...)
+                new_key = []
+                for i, component in enumerate(key):
+                    if isinstance(component, compat.string_types) and \
+                            labels.levels[i].is_all_dates:
+                        new_key.append(slice(component, component, None))
+                    else:
+                        new_key.append(component)
+                key = tuple(new_key)
+
+        return key
+
     def _getitem_axis(self, key, axis=0):
         labels = self.obj._get_axis(axis)
+        key = self._get_partial_string_timestamp_match_key(key, labels)
 
         if isinstance(key, slice):
             self._has_valid_type(key, axis)
@@ -1443,6 +1496,8 @@ class _iLocIndexer(_LocationIndexer):
     - A list or array of integers, e.g. ``[4, 3, 0]``.
     - A slice object with ints, e.g. ``1:7``.
     - A boolean array.
+    - A ``callable`` function with one argument (the calling Series, DataFrame
+      or Panel) and that returns valid output for indexing (one of the above)
 
     ``.iloc`` will raise ``IndexError`` if a requested indexer is
     out-of-bounds, except *slice* indexers which allow out-of-bounds
@@ -1610,6 +1665,12 @@ class _ScalarAccessIndexer(_NDFrameIndexer):
         return self.obj.get_value(*key, takeable=self._takeable)
 
     def __setitem__(self, key, value):
+        if isinstance(key, tuple):
+            key = tuple(com._apply_if_callable(x, self.obj) for x in key)
+        else:
+            # scalar callable may return tuple
+            key = com._apply_if_callable(key, self.obj)
+
         if not isinstance(key, tuple):
             key = self._tuplify(key)
         if len(key) != self.obj.ndim:
@@ -1718,11 +1779,12 @@ def convert_to_index_sliceable(obj, key):
         if key in obj._data.items:
             return None
 
-        # we need a timelike key here
+        # We might have a datetimelike string that we can translate to a
+        # slice here via partial string indexing
         if idx.is_all_dates:
             try:
                 return idx._get_string_slice(key)
-            except:
+            except (KeyError, ValueError, NotImplementedError):
                 return None
 
     return None
@@ -1750,12 +1812,13 @@ def check_bool_indexer(ax, key):
     result = key
     if isinstance(key, ABCSeries) and not key.index.equals(ax):
         result = result.reindex(ax)
-        mask = com.isnull(result._values)
+        mask = isnull(result._values)
         if mask.any():
             raise IndexingError('Unalignable boolean Series key provided')
-
         result = result.astype(bool)._values
-
+    elif is_sparse(result):
+        result = result.to_dense()
+        result = np.asarray(result, dtype=bool)
     else:
         # is_bool_indexer has already checked for nulls in the case of an
         # object array key, so no check needed here
@@ -1892,9 +1955,9 @@ def _non_reducing_slice(slice_):
 
     def pred(part):
         # true when slice does *not* reduce
-        return isinstance(part, slice) or com.is_list_like(part)
+        return isinstance(part, slice) or is_list_like(part)
 
-    if not com.is_list_like(slice_):
+    if not is_list_like(slice_):
         if not isinstance(slice_, slice):
             # a 1-d slice, like df.loc[1]
             slice_ = [[slice_]]
